matrix WorldMatrix;
matrix ViewMatrix;
matrix ProjectionMatrix;

float4 LightDirection[2];
float4 LightColour[2];
float4 PixelOutputColour;

struct VERTEXSHADER_INPUT
{
    float4 Loc : POSITION;
    float3 Norm : NORMAL;
};

struct PIXELSHADER_INPUT
{
    float4 Loc : SV_POSITION;
    float3 Norm : TEXCOORD0;
};


PIXELSHADER_INPUT LightingVertexShader(VERTEXSHADER_INPUT IN)
{
    PIXELSHADER_INPUT output = (PIXELSHADER_INPUT)0;
   
    output.Loc = mul(IN.Loc, WorldMatrix);
    output.Loc = mul(output.Loc, ViewMatrix);
    output.Loc = mul(output.Loc, ProjectionMatrix);
    
    output.Norm = mul(IN.Norm, WorldMatrix);
    
    return output;
}

float4 LightingPixelShader(PIXELSHADER_INPUT IN) : SV_Target
{
    float4 finalPixelColour = 0;
    float4 dotPixelColour = 0;
    
	/* calculate the overall lighting by multiplying the dot product result of the incoming light vector and the
	   surface normal with the color of each light source */
	for(int i = 0; i < 2; i++)
	{
		dotPixelColour = dot((float3)LightDirection[i], IN.Norm);
		finalPixelColour += saturate(dotPixelColour * LightColour[i]);
	}
    
    /* return the overall pixel color */
    return finalPixelColour;
}

float4 RenderColour(PIXELSHADER_INPUT IN) : SV_Target
{
    return PixelOutputColour;
}

technique10 ComputeLighting
{
    pass P0
    {
        SetVertexShader(CompileShader(vs_4_0, LightingVertexShader()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, RenderColour()));
    }
}

technique10 LightScene
{
    pass P0
    {
        SetVertexShader(CompileShader( vs_4_0, LightingVertexShader()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader( ps_4_0, LightingPixelShader()));
    }
}