#include <GL/glut.h>

/*
=================
display
-renders the scene
=================
*/
void display(void)
{
	/* clear frame buffer and z-buffer - thus clearing the screen */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	/* move to the center of the screen */
    glLoadIdentity();

	/* enables lighting  without it entire scene appears dark */
	glEnable(GL_LIGHTING);

	/* activate light x (GL_LIGHTx) in the evaluation of the lighting equation
	   - can be any # but logical numbering helps...*/
	glEnable(GL_LIGHT1);
	/* specify the colour of the light (blue) */
	GLfloat diffuse_light_colour[] = {1.0f, 1.0f, 0.0f};
	/* specify the position of the light */
	GLfloat diffuse_light_position[] = {20.0, 20.0, -20.0, 1.0f};

	/* sets the light source parameters */
	glLightfv( GL_LIGHT1, GL_DIFFUSE, diffuse_light_colour);
	/* sets the light source position */
	glLightfv( GL_LIGHT1, GL_POSITION, diffuse_light_position);

	/* rotate camera 120 degrees about the y-axis */
	glRotatef(120, 0.0, 1.0, 0.0);
	/*  translate (move) camera -1 units (down) on y-axis */
	glTranslatef(0, -1, 0);

	/* define the warehouse floor */
	glBegin(GL_QUADS);
		/* the x, y & z coordinates of the floor's normal */
		glNormal3f(0, 1, 0);
		/* draw the warehouse floor */
		glVertex3f(-20.0, 0.0, -20.0);
		glVertex3f(-20.0, 0.0, 20.0);
		glVertex3f(20.0, 0.0, 20.0);
		glVertex3f(20.0, 0.0, -20.0);
	glEnd();

	/* place our spheres - each glTranslatef call setups the sphere at a different
	   pos */

	/* positions the camera to look at the part of our floor with the spheres */
	glRotatef(135, 0.0, 1.0, 0.0);

	/* gluSphere(quadric object, sphere's radius, longitude subdivisions,
	             latitude subdivisions */

	/* smoothe shaded far right sphere */
	glTranslatef(-9, 1, -1);
	/* create the quadrics object */
	GLUquadricObj*  q = gluNewQuadric();
	/* sets the quadrics' draw style */
	gluQuadricDrawStyle(q, GLU_FILL);
	/* affects the overall shading, either flat or smooth */
	gluQuadricNormals(q, GLU_SMOOTH);
	/* sets texturing of the quadric enabled or disabled */
	gluQuadricTexture(q, GL_FALSE);
	gluSphere(q, 1.0, 20, 16);

	/* flat shaded middle sphere */
	glTranslatef(-1, 1, 4);
	GLUquadricObj*  r = gluNewQuadric();
	gluQuadricDrawStyle(r, GLU_FILL);
	gluQuadricNormals(r, GLU_FLAT);
	gluQuadricTexture(r, GL_FALSE);
	gluSphere(r, 2.0, 20, 16);

	/* smooth shaded far left sphere */
	glTranslatef(1, 0, 5);
	GLUquadricObj*  s = gluNewQuadric();
	gluQuadricDrawStyle(s, GLU_FILL);
	gluQuadricNormals(s, GLU_SMOOTH);
	gluQuadricTexture(s, GL_FALSE);
	gluSphere(s, 2.0, 20, 16);

	/* make sure all previously issued commands are completed */
	glFlush();

	/* contents of back buffer are set to become that of front buffer */
	glutSwapBuffers();
}

/*
================
reshape
-Rendered window's reshape function - manages changes to size of the window, correctly adjusting aspect ratio.
=================
*/
void reshape(int w, int h)
{
	/* sets the viewport with the first two parameters the lower-left corner of the viewport
	   rectangle, in pixels and the last two, the width and height of the viewport */
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);

	/* specifies the Projection matrix as the current matrix */
	glMatrixMode(GL_PROJECTION);
	/* move to the center of the screen */
	glLoadIdentity();

	/* sets up the  perspective projection matrix:
	   - the first parameter specifies the field of view (fov), an angle in degrees
	     (in the y-direction)
	   - the second parameter is aspect ratio determining fov
		 (in the x-direction)
	   - the third parameter is the distance from the viewer to the near clipping plane
	   - the last parameter is the distance from the viewer to the far clipping plane */
	gluPerspective(45, (GLdouble)w / (GLdouble)h, 0.05, 100);
	/* specifies the Modelview matrix as the current matrix */
	glMatrixMode(GL_MODELVIEW);
}

/*
=================
myInit
-handles basic initialisation and setup of the OpenGL parameters
=================
*/
void myInit(void)
{
	/* sets either flat or smooth shading */
	glShadeModel(GL_SMOOTH);
	/* enable hidden-surface removal */
	glEnable(GL_DEPTH_TEST);
	/* cubeSquare culling enabled */
	glEnable(GL_CULL_FACE);
	/* enable normal normalization */
	glEnable(GL_NORMALIZE);
	/* specifies front- and back-facing polygons
	   -counterclockwise polygons are front-facing (GL_CCW)
	   -clockwise polygons are front-facing (GL_CW) */
	glFrontFace(GL_CCW);
	/* specifies whether front- or back-faces are to be culled */
    glCullFace(GL_BACK);
}

/*
=================
main
=================
*/
int main(int argc, char** argv)
{
	/* initialize the GLUT library. */
	glutInit(&argc, argv);
	/* sets the initial display mode: double buffering and a depth buffer. */
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	/* sets default window size */
	glutInitWindowSize(800, 600);
	/* creates a top-level window with a caption */
	glutCreateWindow("...lighting...");
	/* reshape callback for the current window */
	glutReshapeFunc(reshape);
	/* displays our contents in the window */
	glutDisplayFunc(display);
	myInit();
	/* enters the GLUT event processing loop - will handle any callback */
	glutMainLoop();
}
//EOF