#include <windows.h>
#include <d3d10.h>
#include <d3dx10.h>
#include "resource.h"

ID3D10EffectMatrixVariable* g_pd3d10EffectWorldMatrix = NULL;
ID3D10EffectMatrixVariable* g_pd3d10EffectViewMatrix = NULL;
ID3D10EffectMatrixVariable* g_pd3d10EffectProjectMatrix = NULL;

ID3D10EffectVectorVariable* g_pd3d10LightDirectionVectorVar = NULL;
ID3D10EffectVectorVariable* g_pd3d10LightColourVectorVar = NULL;
ID3D10EffectVectorVariable* g_pd3d10OutputColourVectorVar = NULL;

D3DXMATRIX g_ProjectionMatrix;
D3DXMATRIX g_WorldMatrix;
D3DXMATRIX g_ViewMatrix;

D3DXVECTOR4 IncomingLightVector[2];
D3DXVECTOR4 IncomingLightColor[2];

/* create a structure to store a cubes vertex coordinates */
struct CubeVertex
{
    D3DXVECTOR3 Location;  
    D3DXVECTOR3 Normal; 
};

/* initialize the instance handle, g_hInstance,
   the handle to the window, g_hWindow, etc. */
HINSTANCE g_hInstance = NULL;  
HWND g_hWindow = NULL;
D3D10_DRIVER_TYPE g_dx10DriverType = D3D10_DRIVER_TYPE_NULL;
ID3D10Device* g_pd3d10Device = NULL;
IDXGISwapChain* g_pdx10SwapChain = NULL;
ID3D10Texture2D* g_pd3d10DepthStencil = NULL;
ID3D10DepthStencilView* g_pd3d10DepthStencilView = NULL;
ID3D10Effect* g_pd3d10Effect  = NULL;
ID3D10EffectTechnique* g_pd3d10EffectTechniqueLightRenderer = NULL;
ID3D10EffectTechnique* g_pd3d10EffectTechniquePixelLighting = NULL;
ID3D10InputLayout* g_pd3d10InputVertexLayout = NULL;
ID3D10Buffer* g_pd3d10VertexBuffer = NULL;
ID3D10Buffer* g_pd3d10IndexBuffer = NULL;
ID3D10RenderTargetView* g_pd3d10RenderTargetView = NULL;

/* function prototypes */
HRESULT InitDevice();
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
HRESULT InitWindow(HINSTANCE hInstance, int nCmdShow);
void DrawScene();
void Clean();

/* initialize the message processing loop elements and the idle time required
   for the rendering of our triangle via winMain, the application entry point */
int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow)
{
	if((FAILED(InitWindow(hInstance, nCmdShow)))||(FAILED(InitDevice())))
	{
		Clean(); //do device cleanup  freeing resources
		return 0;
	}

	MSG msg = {0};
	while(WM_QUIT != msg.message)
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			DrawScene(); //render the cubes
		}
	}
	Clean();
	return (int)msg.wParam;
}

/* register the class and create the output window */
HRESULT InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	/* register the window class */
	///////////////////////////////
	WNDCLASSEX wndcex;
	wndcex.cbSize 			= sizeof(WNDCLASSEX);
	wndcex.style 			= CS_HREDRAW | CS_VREDRAW;
	wndcex.lpfnWndProc 		= WndProc;
	wndcex.cbClsExtra 		= 0;
	wndcex.cbWndExtra 		= 0;
	wndcex.hInstance 		= hInstance;
	wndcex.hIcon 			= LoadIcon(hInstance, (LPCTSTR)IDI_PROG9_4);
	wndcex.hCursor 			= LoadCursor(NULL, IDC_ARROW);
	wndcex.hbrBackground 	= (HBRUSH)(COLOR_WINDOW+1);
	wndcex.lpszMenuName 	= NULL;
	wndcex.lpszClassName 	= "LitCubes";
	wndcex.hIconSm 			= LoadIcon(wndcex.hInstance, (LPCTSTR)IDI_PROG9_4);

	if(!RegisterClassEx(&wndcex))
		return E_FAIL;

	/* create the window */
	///////////////////////
	g_hInstance = hInstance;

	RECT rectangle = {0, 0, 800, 600};
	AdjustWindowRect(&rectangle, WS_OVERLAPPEDWINDOW, FALSE);

	g_hWindow = CreateWindow(
					"LitCubes",
					"Direct3D 10 Basic Lighting Example",
					WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT, CW_USEDEFAULT,
					rectangle.right - rectangle.left,
					rectangle.bottom - rectangle.top,
					NULL, NULL, hInstance, NULL);

	if(!g_hWindow)
		return E_FAIL;

	ShowWindow(g_hWindow, nCmdShow);

	return S_OK;
}

/* create the Direct3D device and swap chain */
HRESULT InitDevice()
{
    HRESULT hresult_ = S_OK;

	/* setup client rectangle rectangle_width and rectangle_width */
	/////////////////////////////////////////////
	RECT rectangle;
	GetClientRect(g_hWindow, &rectangle);

	int rectangle_width = rectangle.right - rectangle.left;
	int rectangle_height = rectangle.bottom - rectangle.top;
	int deviceFlags = 0;

	/* specify D3D10 drivers to use */
	/////////////////////////////////
    D3D10_DRIVER_TYPE d3d10drivers[] = 
    {
        D3D10_DRIVER_TYPE_HARDWARE,
        D3D10_DRIVER_TYPE_REFERENCE,
    };

	int numberDrivers =	sizeof(d3d10drivers)/sizeof(d3d10drivers[0]);

	/* create the swap chain */
	///////////////////////////
    DXGI_SWAP_CHAIN_DESC swapchain;
	SecureZeroMemory(&swapchain, sizeof(swapchain));

	swapchain.BufferCount = 1;
	swapchain.BufferDesc.Width = rectangle_width;
	swapchain.BufferDesc.Height = rectangle_height;
	swapchain.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	swapchain.BufferDesc.RefreshRate.Numerator = 30;
	swapchain.BufferDesc.RefreshRate.Denominator = 1;
	swapchain.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
	swapchain.OutputWindow = g_hWindow;
	swapchain.SampleDesc.Count = 1;
	swapchain.SampleDesc.Quality = 0;
	swapchain.Windowed = TRUE;

	for(int d3d10DriverCount = 0; d3d10DriverCount < numberDrivers; d3d10DriverCount++)
	{
		g_dx10DriverType = d3d10drivers[d3d10DriverCount];
		hresult_ = D3D10CreateDeviceAndSwapChain(
						NULL, g_dx10DriverType, NULL,
						deviceFlags, D3D10_SDK_VERSION,
						&swapchain, &g_pdx10SwapChain,
						&g_pd3d10Device);

		if(SUCCEEDED(hresult_))
			break;
	}

	if(FAILED(hresult_))
		return hresult_;

/* specify the render target view */
	////////////////////////////////////
	ID3D10Texture2D *pBuffer;
	hresult_ = g_pdx10SwapChain->GetBuffer(0, __uuidof(ID3D10Texture2D),(LPVOID*) &pBuffer);

	if(FAILED(hresult_))
		return hresult_;

	hresult_ = g_pd3d10Device->CreateRenderTargetView(pBuffer, NULL, &g_pd3d10RenderTargetView);

	pBuffer->Release();

	if(FAILED(hresult_))
		return hresult_;

	g_pd3d10Device->OMSetRenderTargets(1,&g_pd3d10RenderTargetView, NULL);

	/* create the viewport */
	/////////////////////////
	D3D10_VIEWPORT viewport;
	viewport.TopLeftX 	= 0;
	viewport.TopLeftY 	= 0;
	viewport.Width 		= rectangle_width;
	viewport.Height 	= rectangle_height;
	viewport.MinDepth 	= 0.0f;
	viewport.MaxDepth 	= 1.0f;

	g_pd3d10Device->RSSetViewports(1, &viewport);

    /* define a depth-stencil texture*/
	///////////////////////////////////
    D3D10_TEXTURE2D_DESC depthResource;
    depthResource.Width = rectangle_width;
    depthResource.Height = rectangle_height;
	depthResource.CPUAccessFlags = 0;
    depthResource.MiscFlags = 0;
    depthResource.MipLevels = 1;
    depthResource.ArraySize = 1;
    depthResource.SampleDesc.Quality = 0;
	depthResource.SampleDesc.Count = 1;
    depthResource.Usage = D3D10_USAGE_DEFAULT;
    depthResource.BindFlags = D3D10_BIND_DEPTH_STENCIL;
    depthResource.Format = DXGI_FORMAT_D32_FLOAT;
   
	hresult_ = g_pd3d10Device->CreateTexture2D(
									&depthResource, 
									NULL, 
									&g_pd3d10DepthStencil);
    
	if(FAILED(hresult_))
        return hresult_;

    /* depth-stencil view */
	////////////////////////
    D3D10_DEPTH_STENCIL_VIEW_DESC depthstencilviewDescription;
    
	depthstencilviewDescription.Format = depthResource.Format;
	depthstencilviewDescription.Texture2D.MipSlice = 0;
    depthstencilviewDescription.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;

    hresult_ = g_pd3d10Device->CreateDepthStencilView(
									g_pd3d10DepthStencil, 
									&depthstencilviewDescription, 
									&g_pd3d10DepthStencilView);
    
	if(FAILED(hresult_))
        return hresult_;

    g_pd3d10Device->OMSetRenderTargets(1, &g_pd3d10RenderTargetView, g_pd3d10DepthStencilView);

	/* load the effect file */
	//////////////////////////
	hresult_ = D3DX10CreateEffectFromFile(
					"effect.fx", NULL, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, 
                    g_pd3d10Device, NULL, NULL, &g_pd3d10Effect, NULL, NULL);

	/* if the effect file is missing */
	///////////////////////////////////
	if(FAILED(hresult_))
	{
		return hresult_;
	}

	/* set the effect techniques */
	///////////////////////////////
	g_pd3d10EffectTechniqueLightRenderer = g_pd3d10Effect ->GetTechniqueByName("LightScene");
    g_pd3d10EffectTechniquePixelLighting = g_pd3d10Effect ->GetTechniqueByName("ComputeLighting");

	/* get the ID3D10EffectMatrixVariable variables */
    g_pd3d10EffectWorldMatrix = g_pd3d10Effect->GetVariableByName("WorldMatrix")->AsMatrix();
    g_pd3d10EffectViewMatrix = g_pd3d10Effect->GetVariableByName("ViewMatrix")->AsMatrix();
    g_pd3d10EffectProjectMatrix = g_pd3d10Effect->GetVariableByName("ProjectionMatrix")->AsMatrix();
    g_pd3d10OutputColourVectorVar = g_pd3d10Effect ->GetVariableByName("PixelOutputColour")->AsVector();

	/* obtain the LightColour shader variable */
	g_pd3d10LightColourVectorVar = g_pd3d10Effect ->GetVariableByName("LightColour")->AsVector();
    	
	/* obtain the LightDirection shader variable */
	g_pd3d10LightDirectionVectorVar = g_pd3d10Effect ->GetVariableByName("LightDirection")->AsVector();

    /* input-inputlayout */
	//////////////////
    D3D10_INPUT_ELEMENT_DESC inputlayout[] =
    {
        { 
			"POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, 
			D3D10_INPUT_PER_VERTEX_DATA, 0 
		},  
        { 
			"NORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, 
			D3D10_INPUT_PER_VERTEX_DATA, 0 
		}, 
    };
    
	int numberOfElements = sizeof(inputlayout) / sizeof(inputlayout[0]);

    /* specify the input-layout object */
    D3D10_PASS_DESC PassDescription;
	g_pd3d10EffectTechniqueLightRenderer->GetPassByIndex(0)->GetDesc(&PassDescription);

	hresult_ = g_pd3d10Device->CreateInputLayout(
								inputlayout, numberOfElements,
								PassDescription.pIAInputSignature,
								PassDescription.IAInputSignatureSize,
								&g_pd3d10InputVertexLayout);

	if(FAILED(hresult_))
		return hresult_;

	/* set the input-layout */
	g_pd3d10Device->IASetInputLayout(g_pd3d10InputVertexLayout);

	/* specify a cubes vertices */
	///////////////////////////////
    CubeVertex array_of_vertex_data[] =
    {
		/* face # 1 */
		{	D3DXVECTOR3(-3.0f, 3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, 1.0f, 0.0f)},		//normal
		
		{	D3DXVECTOR3(3.0f, 3.0f, -3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 1.0f, 0.0f)},		//normal
		
		{	D3DXVECTOR3(3.0f, 3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 1.0f, 0.0f)},		//normal
		
		{	D3DXVECTOR3(-3.0f, 3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 1.0f, 0.0f)},		//normal
		
		/* face # 2 */
		{	D3DXVECTOR3(-3.0f, -3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, -1.0f, 0.0f)},	//normal
		
		{	D3DXVECTOR3(3.0f, -3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, -1.0f, 0.0f)},	//normal

		{	D3DXVECTOR3(3.0f, -3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, -1.0f, 0.0f)},	//normal

		{	D3DXVECTOR3(-3.0f, -3.0f, 3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, -1.0f, 0.0f)},	//normal

			/* face # 3 */
		{	D3DXVECTOR3(-3.0f, -3.0f, 3.0f),	//vertex coordinate
			D3DXVECTOR3(-1.0f, 0.0f, 0.0f)},	//normal

		{	D3DXVECTOR3(-3.0f, -3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(-1.0f, 0.0f, 0.0f)},	//normal

		{	D3DXVECTOR3(-3.0f, 3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(-1.0f, 0.0f, 0.0f)},	//normal

		{	D3DXVECTOR3(-3.0f, 3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(-1.0f, 0.0f, 0.0f)},	//normal

		/* face # 4 */
		{	D3DXVECTOR3(3.0f, -3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(1.0f, 0.0f, 0.0f)},		//normal
		
		{	D3DXVECTOR3(3.0f, -3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(1.0f, 0.0f, 0.0f)},		//normal
		
		{	D3DXVECTOR3(3.0f, 3.0f, -3.0f),		//vertex coordinate
			D3DXVECTOR3(1.0f, 0.0f, 0.0f)},		//normal
		
		{	D3DXVECTOR3(3.0f, 3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(1.0f, 0.0f, 0.0f)},		//normal
		
		/* face # 5 */
		{	D3DXVECTOR3(-3.0f, -3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, -1.0f)},	//normal
		
		{	D3DXVECTOR3(3.0f, -3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, -1.0f)},	//normal

		{	D3DXVECTOR3(3.0f, 3.0f, -3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, -1.0f)},	//normal

		{	D3DXVECTOR3(-3.0f, 3.0f, -3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, -1.0f)},	//normal

			/* face # 6 */
		{	D3DXVECTOR3(-3.0f, -3.0f, 3.0f),	//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, 1.0f)},		//normal

		{	D3DXVECTOR3(3.0f, -3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, 1.0f)},		//normal

		{	D3DXVECTOR3(3.0f, 3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, 1.0f)},		//normal

		{	D3DXVECTOR3(-3.0f, 3.0f, 3.0f),		//vertex coordinate
			D3DXVECTOR3(0.0f, 0.0f, 1.0f)},		//normal
    };
	
	/* setup the buffer resource */
	///////////////////////////////
    D3D10_BUFFER_DESC bufferdesc;
    bufferdesc.Usage			= D3D10_USAGE_DEFAULT;
    bufferdesc.BindFlags		= D3D10_BIND_VERTEX_BUFFER;
    bufferdesc.CPUAccessFlags	= 0;
    bufferdesc.MiscFlags		= 0;
	bufferdesc.ByteWidth		= sizeof(CubeVertex)*24;

	/* initialize resource */
	/////////////////////////
    D3D10_SUBRESOURCE_DATA ResourceData;
    ResourceData.pSysMem = array_of_vertex_data;
	hresult_ = g_pd3d10Device->CreateBuffer(
								&bufferdesc,
								&ResourceData,
								&g_pd3d10VertexBuffer);

    if( FAILED(hresult_) )
        return hresult_;

	/* setup the vertex buffer */
	/////////////////////////////
	UINT elementstride = sizeof(CubeVertex);

	UINT bufferoffset = 0;
	g_pd3d10Device->IASetVertexBuffers(
						0, 1, &g_pd3d10VertexBuffer,
						&elementstride, &bufferoffset);

    /* index array */
    UINT index_array[] =
    {
        3, 1, 0, 2, 1, 3, 6, 4, 5, 
		7, 4, 6, 11, 9, 8, 10, 9, 11, 
		14, 12, 13, 15, 12, 14, 19, 17, 16, 
		18, 17, 19, 22, 20, 21, 23, 20, 22
    };

	bufferdesc.BindFlags = D3D10_BIND_INDEX_BUFFER;
    bufferdesc.Usage = D3D10_USAGE_DEFAULT;
    bufferdesc.ByteWidth = sizeof(DWORD)*36;
 
    ResourceData.pSysMem = index_array;
    
	g_pd3d10Device->CreateBuffer(&bufferdesc, &ResourceData, &g_pd3d10IndexBuffer);
    g_pd3d10Device->IASetIndexBuffer(g_pd3d10IndexBuffer, DXGI_FORMAT_R32_UINT, 0);

	/* bind the primitive type and data order information to a
	   triangle list */
	g_pd3d10Device->IASetPrimitiveTopology(
						D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	/* initialize the world matrix to an identity matrix */
	///////////////////////////////////////////////////////
	D3DXMatrixIdentity(&g_WorldMatrix);

	/* set the view matrix */
	///////////////////////////////////////////////////////
	D3DXVECTOR3 EyeCoord( 7.0f, 6.0f, -13.0f);
	D3DXVECTOR3 LookAt(0.0f, 1.0f, 0.0f);
	D3DXVECTOR3 UpDir(0.0f, 4.0f, 0.0f);
	D3DXMatrixLookAtLH(&g_ViewMatrix, &EyeCoord, &LookAt, &UpDir);

 	/* set the left-handed perspective projection */
	////////////////////////////////////////////////
	D3DXMatrixPerspectiveFovLH(&g_ProjectionMatrix, (float)D3DX_PI*0.25f, rectangle_width/rectangle_height, 0.1f, 100.0f);

    return S_OK;
}

/* setup our WndProc callback */
LRESULT CALLBACK WndProc(HWND hWindow, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC devicecontext;
	PAINTSTRUCT paintstructure;

	switch(msg)
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		case WM_PAINT:
			devicecontext = BeginPaint (hWindow, &paintstructure);
			EndPaint(hWindow, &paintstructure);
		break;

	default:
		return DefWindowProc(hWindow, msg, wParam, lParam);
	}
	return 0;
}

/* checks whether an object has been created, and if it has, deallocates it */
void Clean()
{
	if(g_pd3d10Device)
		g_pd3d10Device->ClearState();

	if(g_pd3d10InputVertexLayout)
		g_pd3d10InputVertexLayout->Release();

	if(g_pd3d10VertexBuffer)
		g_pd3d10VertexBuffer->Release();

	if(g_pd3d10IndexBuffer) 
		g_pd3d10IndexBuffer->Release();

	if(g_pd3d10RenderTargetView)
		g_pd3d10RenderTargetView->Release();

	if(g_pd3d10DepthStencilView) 
		g_pd3d10DepthStencilView->Release();

	if(g_pd3d10Effect)
		g_pd3d10Effect->Release();

	if(g_pd3d10Device)
		g_pd3d10Device->Release();

	if(g_pdx10SwapChain)
		g_pdx10SwapChain->Release();
}

int run_once = 0;

/* draw the cube */
void DrawScene()
{
	/* set the lighting position and color arrays */
	/* initialize the direction of each parallel light source */
    D3DXVECTOR4 IncomingLightVector[2] = 
    { 
		//the spatial position of the first light source
		D3DXVECTOR4(1.0f, 0.5f, 0.5f, 1.0f),
		//the spatial position of the second light source
		D3DXVECTOR4(0.0f, 0.0f, 1.0f, 1.0f)
    };
    
	/* specify the color of each parallel light source */
	D3DXVECTOR4 IncomingLightColour[2] = 
    {
		//bright red
		D3DXVECTOR4(1.0f, 0.0f, 0.0f, 1.0f),
		//deep orange
		D3DXVECTOR4(1.0f, 0.5f, 0.0f, 1.0f)
    };

	if(run_once == 0)
	{
		g_pd3d10EffectViewMatrix->SetMatrix((float*)&g_ViewMatrix);
		g_pd3d10EffectProjectMatrix->SetMatrix((float*)&g_ProjectionMatrix);

		g_pd3d10LightDirectionVectorVar->SetFloatVectorArray((float*)IncomingLightVector, 0, 2);
		g_pd3d10LightColourVectorVar->SetFloatVectorArray((float*)IncomingLightColour, 0, 2);
	}

	/* start by clearing the back-buffer with the color white */
	float ClearBufferColour[4] = {1.0f, 1.0f, 1.0f, 1.0f};
	g_pd3d10Device->ClearRenderTargetView(g_pd3d10RenderTargetView, ClearBufferColour);

	/* update the world matrix */
	/////////////////////////////
    g_pd3d10EffectWorldMatrix->SetMatrix((float*)&g_WorldMatrix);

	/* set the depth buffer's depth */
    g_pd3d10Device->ClearDepthStencilView(g_pd3d10DepthStencilView, D3D10_CLEAR_DEPTH, 1.0f, 0 );

	/* obtain the D3D10_TECHNIQUE_DESC effect-variable description */
	D3D10_TECHNIQUE_DESC technique;
	g_pd3d10EffectTechniqueLightRenderer->GetDesc(&technique);

	/* render by looping over the number of technique passes */
	for(int i = 0; i < technique.Passes; ++i)
	{
		g_pd3d10EffectTechniqueLightRenderer->GetPassByIndex(i)->Apply(0);
		g_pd3d10Device->DrawIndexed(36, 0, 0);
	}

	/* render the lights */
	///////////////////////

	int light_number = 0;
	
	while (light_number < 2)
	{
		light_number = light_number + 1;

		/* update the world matrix to include the current light */
        g_pd3d10OutputColourVectorVar->SetFloatVector((float*)&IncomingLightColour[light_number]);

		/* obtain the D3D10_TECHNIQUE_DESC effect-variable description */
		D3D10_TECHNIQUE_DESC technique;
		g_pd3d10EffectTechniqueLightRenderer->GetDesc(&technique);

		/* render by looping over the number of technique passes */
		for(int i = 0; i < technique.Passes; ++i)
		{
			g_pd3d10EffectTechniqueLightRenderer->GetPassByIndex(i)->Apply(0);
			g_pd3d10Device->DrawIndexed(36, 0, 0);
		}
     }

	/* switch the back- and front-buffer and display the
	   triangle */
	g_pdx10SwapChain->Present(0,0);
	
	run_once = 1;
}